# -*- coding: future_fstrings -*-
from __future__ import unicode_literals

import cbor2
from cbor2 import CBORTag,encoder

from . import bindings
import codecs
import string

# ugly patch to remove default UUID decodeur
cbor2.decoder.semantic_decoders.pop(37)

#Py2 is running future_fstrings or not
try:
    codecs.lookup('future_fstrings')
    NEED_UNICODE_FIX=False
except LookupError:
    NEED_UNICODE_FIX=True


def tag_hook(decoder, tag, shareable_index=None):
    if tag.tag == 37:
        return bindings.UUID(bytes=tag.value)
    if tag.tag == 32:
        return bindings.URL(tag.value)
    return tag

def default_encoder(encoder, value):
    if isinstance(value,bindings.UUID):
        encoder.encode(CBORTag(37, value.bytes))

    if isinstance(value,bindings.URL):
        encoder.encode(CBORTag(32, value.bytes))

def dumps(obj, **kwargs):
    if NEED_UNICODE_FIX:
        fix_unicode(obj)
    return cbor2.dumps(obj,default=default_encoder,**kwargs)

def loads(payload, **kwargs):
    return cbor2.loads(payload,tag_hook=tag_hook,**kwargs)

def fix_unicode(obj):
    """Py2 default string is <str>. Py3 is <unicode>, and even worst u'test=='test' in Py2.
    I decided to leave the default behaviour for each Python version avoid to rewrite a vast
    majority of the code. The downside : I have for monkey fix string before CBOR encoding
    """
    # mostly a cut / paste from cleanup
    if isinstance(obj,list):
        for i in range(0,len(obj)):
            obj[i] = fix_unicode(obj[i]) 
        return obj

    if isinstance(obj,dict):
        result = {}
        for k in obj.keys():
            result.update({fix_unicode(k):fix_unicode(obj[k])})
        return result

    #print("%s %s %s" % (type(obj),str(obj),str_or_bytes(obj)))
    if str_or_bytes(obj):
        return codecs.decode(obj,'utf-8')
    else:
        return obj


def str_or_bytes(data):
    """ return True if data is a unicode_able string """
    if type(data) != str:
        return False
    # bytes array are <str> in Py2, so only deal w/ something I can print
    # I'm unable to find a better way :(
    print_set = set(string.printable)
    data_set = set(data)
    return data_set.issubset(print_set)

def cleanup(obj):
    """ 
    recursive walk a object to search for un-wanted CBOR tags.
    Transform this tag in string format, this can be UUID, URL..
    Should be Ok, with list, dicts.. 
    Warning: This operate in-place changes. 
    Warning: This won't work for tags in dict keys.
    """
    if isinstance(obj,list):
        for i in range(0,len(obj)):
            obj[i] = cleanup(obj[i]) 
        return obj

    if isinstance(obj,dict):
        for k in obj.keys():
            obj.update({k:cleanup(obj[k])})
        return obj

    if obj.__class__ in bindings.classes:
        return str(obj)
    else:
        return obj
